"use strict";
const axios_1 = require("axios");
const mqtt_topping_1 = require("@artcom/mqtt-topping");
const logger_1 = require("@artcom/logger");
async function retrieveBootstrapData(url, timeout, retryDelay, logger, debugBootstrapData) {
    if (debugBootstrapData) {
        logger.info("Using debug bootstrap data", { ...debugBootstrapData });
        return debugBootstrapData;
    }
    else {
        logger.info("Querying bootstrap data", { url });
        while (true) { // eslint-disable-line no-constant-condition
            try {
                const { data } = await axios_1.default.get(url, { timeout });
                logger.info("Bootstrap data received", { ...data });
                return data;
            }
            catch (error) {
                logger.error(`Query failed. Retrying in ${retryDelay}ms...`, { error: error.message });
                await delay(retryDelay);
            }
        }
    }
}
function delay(time) {
    return new Promise(resolve => setTimeout(resolve, time));
}
function connectMqttClient(serviceId, { device, tcpBrokerUri }, onParseError, logger) {
    const clientId = createClientId(serviceId, device);
    logger.info("Connecting to Broker", { tcpBrokerUri, clientId });
    const mqttClient = (0, mqtt_topping_1.connect)(tcpBrokerUri, { clientId, onParseError });
    mqttClient.on("connect", () => { logger.info("Connected to Broker"); });
    mqttClient.on("close", () => { logger.error("Disconnected from Broker"); });
    mqttClient.on("error", () => { logger.error("Error Connecting to Broker"); });
    return mqttClient;
}
function createClientId(serviceId, device) {
    const uuid = Math.random().toString(16).substr(2, 8);
    return `${serviceId}-${device}-${uuid}`;
}
function createQueryConfig(configServerUri) {
    return async (configPath, params = {}) => {
        const { version = "master", listFiles = false, includeCommitHash = false, parseJSON = true } = params;
        const query = {
            url: `${configServerUri}/${version}/${configPath}?listFiles=${listFiles}`,
            transformResponse: parseJSON ? undefined : []
        };
        return (0, axios_1.default)(query)
            .then(response => includeCommitHash
            ? { data: response.data, commitHash: response.headers["git-commit-hash"] }
            : response.data);
    };
}
module.exports = async function init(url, serviceId, { timeout = 2000, retryDelay = 10000, debugBootstrapData = null, onParseError = null } = {}) {
    const logger = (0, logger_1.createLogger)();
    const data = await retrieveBootstrapData(url, timeout, retryDelay, logger, debugBootstrapData);
    return {
        logger,
        data,
        mqttClient: connectMqttClient(serviceId, data, onParseError, logger),
        httpClient: new mqtt_topping_1.HttpClient(data.httpBrokerUri),
        queryConfig: createQueryConfig(data.configServerUri),
        unpublishRecursively: mqtt_topping_1.unpublishRecursively
    };
};
//# sourceMappingURL=main.js.map